% This procedure generates results in Table 4 of the paper: Inference for VARs Identified Restrictions, draft January 2013 
clear all

%---    INITIALIZATION
T = 100;
N_sim = 5000; %# samples of artificial data; used also to estimate lambda

nirf_rest = 4; % horizons for which are sign restrictions are imposed; horizon 1 is the contemporaneous responses 
nirf = nirf_rest; %#maximum horizon for irf

seed = 5;
ndrawscv = 100000; %# draws for simulated cv
alpha = 0.1; %overal significance level for theta
alpha1 = 0.05;%significance level confidence set for q
alpha2 = alpha - alpha1; %significance level, conditional confidence set for theta

design = 2; %pick the design

%design of GDP
const = [0  0];

if design == 2
phi = [0.873 0.003; -0.229 0.230];  %design 2 NBER wp Table 1.
sigma = [0.087  -0.027; -0.027 0.640];  %design 2 NBER wp Table 1.
elseif design == 3
phi = [0.806 0.032; -0.278 0.985];  %design 3 NBER wp Table 1.
sigma = [0.080  -0.023; -0.023 0.674];  %design 3 NBER wp Table 1.
elseif design == 4
 phi = [0.450 0.014; 0.060 0.953];  %design 4 NBER wp Table 1.
 sigma = [0.044  -0.0093; -0.009 0.296];  %design 4 NBER wp Table 1.
end

nvar = size(phi,1);

sigmaL = chol(sigma,'lower');
YYirf = estimate_irfs(phi,sigmaL, nirf);
R1 = YYirf(1:nirf_rest,:); % Reduced form impulse response that are sign restricted or part of theta
phivec = reshape(R1',size(YYirf,1)*size(YYirf,2),1);
phivec = [phivec(1,1);phivec(3:end,1)]; %take out the zero element due to the impact IRF
m = size(phivec,1);

%%%%%%%%%%%%    Compute identified set for q   %%%%%%%%%%%%%%%%%%%
disp('************************');
disp('Computing Identified Set for q ');
disp('                        ');

stepsize = 0.01;
angle = [-pi:stepsize:pi];

[IS, idq, q_binary] = setestimatorv2(phivec, angle); %identified set

temp_q = [1:1:size(angle,2)]'.*q_binary';
temp_qset = temp_q(temp_q~=0);
minqid = min(temp_qset);
maxqid = max(temp_qset);
clear temp_q temp_qset

emptyq = 0;
kappaT = 1.96*log(log(T));

    for nsim = 1:N_sim
        
       fprintf('Iteration = %d\n',nsim);
       
        %% (1) generate data
        [Y] =  gdp_VAR(const,phi,sigma,T,seed,nsim); %generate data        
       
        Zm = randn(m,500); %used to compute cv
        
        %  (2) estimate VAR(1) no constant and construct vector phi
        yy = Y(2:T,:);
        xx = Y(1:T-1,:);
        phi_hat = yy'*xx*inv(xx'*xx);
        resid = yy - xx*phi_hat';
        l = ones(T-1,1);
        M_l = eye(T-1) - l*inv(l'*l)*l';
        sigma_hat = resid'*M_l*resid/(T-2);
        clear yy xx resid
        sigmaL_hat = chol(sigma_hat,'lower');
        YYirf_hat = estimate_irfs(phi_hat,sigmaL_hat, nirf); %obtain IRF
        R1_hat = YYirf_hat(1:nirf_rest,:);
        phihatvec = reshape(R1_hat', size(YYirf_hat,1)*size(YYirf_hat,2), 1);
        phihatvec = [phihatvec(1,1);phihatvec(3:end,1)];%take out the zero element due to the impact IRF
        

        %   (3) estimate lambda
        lambda_hat = est_lambdav2(const, phi_hat, sigma_hat, T, seed, N_sim, phihatvec,nirf_rest, nirf);
        Lhat = chol(lambda_hat,'lower');
              
        q1_vec = cos(angle);
        q2_vec = sin(angle);
      
        % (4) compute Bonferroni CS 
        [CSBI(:,nsim), CSBIq_binary(nsim,:), lengthq(nsim,:), bindq(nsim,:), emptyq] = cs_bonferroni(Zm,phihatvec,lambda_hat,q1_vec,q2_vec,nvar,T, kappaT, alpha1, alpha2, emptyq);
        
        if emptyq(1,1) == 1
            emptyq = [emptyq, nsim];
        end
        
        clear  phihatvec   
        
    end %end of loop for nsim

if size(emptyq,2)>= 2
  CSBI(:,emptyq(1,2:end))=[];
  lengthq(emptyq(1,2:end),:) = [];
end

tempmomBI = (CSBI(2,:)-(IS(2,1)*ones(1,N_sim - (size(emptyq,2)-1))) > 0);
cov_probBI = (sum(tempmomBI))/N_sim;
lengthBI = mean(CSBI(2,:)-CSBI(1,:));
lengthCSq = mean(lengthq);%to be multiplied by pi
cove_q = mean(CSBIq_binary);
covq = cove_q(1,minqid);
bindq = mean(bindq);

clear cove_q

Table = [
    {'Table ','Coverage', 'Length ','Bind. Ineq'};
    'ID set(q)', 'NA', num2cell(idq(1,end)-idq(1,1)),'NA';
    'CS(q)', num2cell(covq) , num2cell(lengthCSq), num2cell(bindq(1,minqid));
    'ID set (theta)',  'NA', num2cell(IS(2,1)-IS(1,1)),'NA';
    'CS(theta)', num2cell(cov_probBI), num2cell(lengthBI),'NA'
    ];

save ( ['WorkspaceNobs' num2str(T) 'Nobs' num2str(N_sim) 'h' num2str(nirf_rest) 'design' num2str(design)]) 

