%procedure to compute the CS for theta and q
function [CSB, q_CS, length_q, bindq, emptyq] = cs_bonferroni(Zm,phihatvec,lambda_hat,q1_vec,q2_vec,nvar,T, kappaT, alpha1, alpha2, emptyq)

m = size(phihatvec,1);
Lhat = chol(lambda_hat,'lower');
rr = 1;
bindq = zeros(1, size(q1_vec,2));
CSBq  = [];
q_CS = zeros(1, size(q1_vec,2));
angle = [-pi:0.01:pi];

for s = 1:size(q1_vec,2)
    q1 = q1_vec(1,s);
    q2 = q2_vec(1,s);
    %%Sq will change if you restrict the horizon after the impact
    Sq = [[q1, zeros(1,m-1)];[zeros((m-1)/nvar,1),kron(eye((m-1)/nvar),[q1 q2])]]; %it considers all the sign restrictions S(q)
    Shat = Sq*lambda_hat*Sq';
    Dhat = diag(diag(Shat)); %diagonal matrix
    l = size(Sq,1); %row dimention of Sq, size of mu
    BhatI = eye(l);
    
    % BONFERRONI CS
    %%%% step 0: compute objective function for Bonferroni
    
    Z = Sq*phihatvec;
    muI =  (Z>=0).*Z;
    Q_RI(1,s) = T* (diag(diag(Dhat).^(-1/2))*(Z-muI))'*BhatI*(diag(diag(Dhat).^(-1/2))*(Z-muI));
    
    
    %%%% step 1: compute Bonferroni confidence set for q
    
    csi = sqrt(T)*diag(diag(Dhat).^(-1/2))*Sq*phihatvec;
    bind = (csi<kappaT); %indicator which takes value 1 if the inequality is considered binding;
    br = sum(bind);
    bindq(1,s) = br;
    
    for ff = 1:size(Zm,2)
        
        A = diag(diag(Dhat(bind,bind)).^(-1/2))*Sq(bind,:)*Lhat*Zm(:,ff); % first term in the minimization problem
        
        uI =  (A>=0).*A;
        
        GbarI(1,ff) = (A-uI)'*BhatI(bind,bind)*(A-uI);
        
    end
    
    cvBI(1,s)= prctile(GbarI,(1-alpha1)*100);
    
       
    % step 2: conditional on q construct the (1-alpha2) confidence set for theta
    
    if Q_RI(1,s) - cvBI(1,s)<= 0
        q_CS(1,s) = 1;
        CSBq = [CSBq, angle(1,s)];
        CSI(1,rr) = max(0,q1*phihatvec(1,1) -(norminv(1-alpha2/2,0,1))*q1*(lambda_hat(1,1)/T)^(1/2));
        CSI(2,rr) = q1*phihatvec(1,1)+(norminv(1-alpha2/2,0,1))*q1*(lambda_hat(1,1)/T)^(1/2);
        
        rr = rr+1;
    end
    
end
if sum(q_CS)>= 1
    length_q = max(CSBq)- min(CSBq);
    CSB = [min(CSI(1,:)), max(CSI(2,:))];
    emptyq(1,1) = 0;
else
    length_q = 0;
    CSB = [0, 0];
    emptyq(1,1) = 1;
end