% This procedure generates the results in Table 3, first two columns of the paper Inference for SVARs Identified with Sign Restrictions, draft 2017. 

%---    INITIALIZATION
T = 100;
N_sim = 5000;

seed = 2.453745165;
seedcv = 3.85668;
ndrawscv = 100000; %# draws for simulated cv
alpha = 0.1; %overal significance level for theta
alpha1 = 0.05; %significance level confidence set for q
alpha2 = alpha-alpha1; %significance level, conditional confidence set for theta

%set-up for minim problem
Aeq = [];
beq = [];
lb = [];
ub = [];
nonlcon = [];
options = optimset('Display', 'off') ;

    %CRITICAL VALUES:
    randn('state',seedcv);
    z = randn(1,ndrawscv);
    indexn = (z<0); %indicator function: takes value one when z is negative
    cvB1 = prctile(indexn.*(z.^2),(1-alpha1)*100);
    cvB2 = norminv(1-alpha2/2,0,1);
    clear indexn 
   
    kappaT = 1.96*log(log(T));
    CS_phi = 0; %initialize counter to compute coverage probability of the CI for the reduced form parameter  

  
%population parameter values
const = zeros(1,2);
phi = zeros(2,2);
sigma = [0.356  -0.122; -0.122 0.701];  %design 1 NBER wp Table 1.
sigmaL = chol(sigma,'lower');
phivec = reshape(sigmaL',size(sigma,1)*size(sigma,2),1);
phivec = [phivec(1,1);phivec(3:4,1)]; %take out the zero element;

%%%%%%%%%%%%    Compute identified set for q and theta   %%%%%%%%%%%%%%%%%%%
disp('************************');
disp('Computing Identified Set for q and theta');
disp('                        ');

stepsize = 0.01; %stepsize of grid for angle, preferred stepsize = 0.01;
[Fq_l,Fq_u] = setestimator_q(phivec, stepsize);
IS_q =[acos(Fq_u), acos(Fq_l)]; %needed to compute lenght of the identified set for q

 if (phivec(2,1) >= 0)
        IS = phivec(1,1)*max(1,sqrt((phivec(3,1)^2)/(phivec(2,1)^2 + phivec(3,1)^2)));       
  else
        IS = phivec(1,1)*sqrt((phivec(3,1)^2)/(phivec(2,1)^2 +phivec(3,1)^2));       
 end
                      

for nsim = 1:N_sim %loop over different generated samples
    
    fprintf('Iteration = %d\n',nsim);
    tic
    
    %% generate data 
    
    [Y] =  gdp_VAR(const,phi,sigma,T,seed,nsim); %generate data (set phi =0 for a VAR(0))
    
    Z3 = randn(3,500);
    
    disp(nsim)
    %estimate VAR(0) no constant
        l = ones(T,1);
        M_l = eye(T) - l*inv(l'*l)*l';
        sigma_hat = Y'*M_l*Y/(T-1);
        sigmaL_hat = chol(sigma_hat,'lower');
        phihatvec = reshape(sigmaL_hat, size(sigma_hat,1)*size(sigma_hat,2), 1);
        phihatvec = [phihatvec(1:2,1);phihatvec(4,1)];
        phihat1 =  phihatvec(1,1);
        phihat2 =  phihatvec(2,1);
        phihat3 =  phihatvec(3,1);
        
        
        %estimate lambda
        lambdahat = est_lambda(const, phi, sigma_hat, T, seed, N_sim, phihatvec);
       
        %compute the coverage probabilities of the CI of the reduced form parameter
        if T*(phihatvec-phivec)'*inv(lambdahat)*(phihatvec-phivec)<= chi2inv(1-alpha,size(phivec,1)) 
            CS_phi = CS_phi+1;
        end
        
    %----  BONFERRONI APPROACH   --------------
    
   %part one of Bonferroni: construct cs_gis of phi
   
   [CSB_O(:,nsim),CSB_I(:,nsim),CSq_O(:,nsim), CSq_I(:,nsim),arc_O(:,nsim),arc_I(:,nsim), bind_q(nsim,:)] = cs_gis(stepsize, lambdahat, phihatvec, T,  kappaT, cvB1, cvB2, Z3, alpha1);

        %% Estimated Identified Set for theta
    if (phihat2 >= 0)
        theta_max_hat = phihat1*max(1,sqrt((phihat3^2)/(phihat2^2 + phihat3^2)));       
    else
        theta_max_hat = phihat1*sqrt((phihat3^2)/(phihat2^2 + phihat3^2));       
    end
    
    IShat(1,nsim) = theta_max_hat;
    
 toc
 
end %end of loop for nsim

CS_phi= CS_phi/N_sim; %coverage probability for the reduced form parameters
 
%coverage probability
temp = (IShat-(IS*ones(1,N_sim)) > 0);
cov_prob = (sum(temp))/N_sim;
lenght = mean(IShat);
n_bind = mean(bind_q);


tempmomBI = (CSB_I(2,:)-(IS*ones(1,N_sim)) > 0);
cov_probBI = (sum(tempmomBI))/N_sim;
lenghtBI = mean(CSB_I(2,:)-CSB_I(1,:));

tempmomBO = (CSB_O(2,:)-(IS*ones(1,N_sim)) > 0);
cov_probBO = (sum(tempmomBO))/N_sim;
lenghtBO = mean(CSB_O(2,:)-CSB_O(1,:));

tempqO_u = (CSq_O-(Fq_u*ones(1,N_sim)) > 0);
cov_probqO = (sum(tempqO_u))/N_sim;
lenghtqO = mean(arc_O(2,:)-arc_O(1,:));

tempqI_u = (CSq_I-(Fq_u*ones(1,N_sim)) > 0);
cov_probqI = (sum(tempqI_u))/N_sim;
lenghtqI = mean(arc_I(2,:)-arc_I(1,:));


Table = [{'Table ','cov prob', 'lenght'};
    'length F(q)', num2cell(IS_q(1,2)-IS_q(1,1)), '_';
    'length F(theta)',num2cell(IS), '_'; 
    'CSq I',num2cell(cov_probqI),num2cell(lenghtqI);
    'Bonf Iden', num2cell(cov_probBI), num2cell(lenghtBI);
    'CSphi',num2cell(CS_phi), '_';
     ]

save ( ['WorkspaceNobs' num2str(T) 'Nsim' num2str(N_sim) 'design' num2str(1)]) 