% This procedure generates the results in Table 3, Design 2-4, of the paper Inference for SVARs Identified with Sign Restrictions, draft 2017. 

%---    INITIALIZATION
T = 100;
N_sim = 5000; %# samples of artificial data; it is also used also to estimate lambda

seed = 5;
ndrawscv = 100000; %# draws for simulated cv
alpha = 0.1; %overal significance level for theta
alpha1 = 0.05; %significance level confidence set for q
alpha2 = alpha - alpha1; %significance level, conditional confidence set for theta

%set-up for minim problem
Aeq = [];
beq = [];
lb = [];
ub = [];
nonlcon = [];
options = optimset('Display', 'off') ;

    %CRITICAL VALUES:
    randn('state',seed); 
    z = randn(1,ndrawscv);
    indexp = (z>=0); %indicator function: takes value one when z is positive
    chi2t = prctile(indexp.*(z.^2),(1-alpha1)*100);
    kappaT = 1.96*log(log(T));
    
    
%design of GDP
const = [0  0];
design = 3;

if design == 2
    phi = [0.873 0.003; -0.229 0.230];  %design 2 NBER wp Table 1.
    sigma = [0.087  -0.027; -0.027 0.640];  %design 2 NBER wp Table 1.
    
elseif design == 3
    phi = [0.806 0.032; -0.278 0.985];  %design 3 NBER wp Table 1.
    sigma = [0.080  -0.023; -0.023 0.674];  %design 3 NBER wp Table 1.
    
elseif design == 4
    phi = [0.450 0.014; 0.060 0.953];  %design 4 NBER wp Table 1.
    sigma = [0.044  -0.0093; -0.009 0.296];  %design 4 NBER wp Table 1.
end

sigmaL = chol(sigma,'lower');
R1 = phi*sigmaL; % Reduced form impulse response
phivec = reshape(R1',size(phi,1)*size(phi,2),1);
IS = setestimator(phivec); %identified set for VAR(1) 

CS_phi = 0; %initialize coverage probability for the reduced form parameters

%%%%%%%%%%%%    Compute identified set for q   %%%%%%%%%%%%%%%%%%%
disp('************************');
disp('Computing Identified Set for q');
disp('                        ');

stepsize = 0.01;
angle = [-pi:stepsize:pi]; %grid for q1 and q2 

[Fq_l,Fq_u] = setestimator_q(phivec,stepsize); 
IS_q =[acos(Fq_u), acos(Fq_l)]; %needed to compute lenght of the identified set for q


    for nsim = 1:N_sim
        fprintf('Iteration = %d\n',nsim);
         tic
        [Y] =  gdp_VAR(const,phi,sigma,T,seed,nsim); %generate data
        
        Z4 = randn(4,500);
        
        %estimate VAR(1) no constant
        yy = Y(2:T,:);
        xx = Y(1:T-1,:);
        phi_hat = yy'*xx*inv(xx'*xx);
        resid = yy - xx*phi_hat';
        l = ones(T-1,1);
        M_l = eye(T-1) - l*inv(l'*l)*l';
        sigma_hat = resid'*M_l*resid/(T-2);
        clear yy xx resid
        sigmaL_hat = chol(sigma_hat,'lower');
        R1_hat = phi_hat*sigmaL_hat;
        phihatvec = reshape(R1_hat', size(phi,1)*size(phi,2), 1);
        
        %estimate identified set
        [IS_hat(:,nsim)] = setestimator(phihatvec);
        
        %estimate lambda
        lambda_hat = est_lambda(const, phi_hat, sigma_hat, T, seed, N_sim, phihatvec);
        Lhat = chol(lambda_hat,'lower');
        
        %compute the coverage probabilities of the CI of the reduced form parameter
        if T*(phihatvec-phivec)'*inv(lambda_hat)*(phihatvec-phivec)<= chi2inv(1-alpha,size(phivec,1)) %compute coverage probability for reduced form parameters
          CS_phi = CS_phi+1;
        end
        
        q1_vec = cos(angle);
        q2_vec = sin(angle);
        ll = 1;
        rr = 1;
            
      for s = 1:size(angle,2)
                %%------------------------------------------------------------
                %BONFERRONI APPROACH%
                %step 1: objective function for moment selection
                S_q = [q1_vec(1,s) q2_vec(1,s) zeros(1,2); zeros(1,2) q1_vec(1,s) q2_vec(1,s)];
                Shat = S_q*lambda_hat*S_q';
                Dhat = diag(diag(Shat));
                                             
                   Z1 = S_q(1,:)*phihatvec;
                   Z2 = S_q(2,:)*phihatvec;
                     
                    Omega = diag( diag(Dhat).^(-1/2))*Shat*diag( diag(Dhat).^(-1/2));
                     
                    BhatO = inv(Omega);
                    BhatI = eye(size(Omega));                                 
                     
                    csi2 = sqrt(T)*(Dhat(2,2)^(-1/2))*(q1_vec(1,s)*phihatvec(3,1) + q2_vec(1,s)*phihatvec(4,1)); %second moment condition

                  %%-------------------------------------------------------------   
                   % BONFERRONI CONFIDENCE SET
                   %%%% step 0: compute objective function for Bonferroni
                   %%%% confidence set for q
                      
                   Z = [Z1;Z2];
                    if Z>= 0
                        muO = Z;
                        muI = Z;
                    else
                       [muO] = fmincon(@(muO)(T* (diag(diag(Dhat).^(-1/2))*(Z-muO))'*BhatO*(diag(diag(Dhat).^(-1/2))*(Z-muO))),[0,0]',-eye(2),[0,0]',Aeq,beq,lb,ub,nonlcon,options);
                       muI = (Z>=0).*Z;                       
                    end
                    
                    Q_RO(1,s) = T* (diag(diag(Dhat).^(-1/2))*(Z-muO))'*BhatO*(diag(diag(Dhat).^(-1/2))*(Z-muO));
                    Q_RI(1,s) = T* (diag(diag(Dhat).^(-1/2))*(Z-muI))'*BhatI*(diag(diag(Dhat).^(-1/2))*(Z-muI));
  
                  
                   %%%% step 1: compute cv for Bonferroni confidence set for q                                         
                    csi1 = sqrt(T)*(Dhat(1,1)^(-1/2))*(q1_vec(1,s)*phihatvec(1,1) + q2_vec(1,s)*phihatvec(2,1));
                   
                    if csi1<kappaT && csi2 < kappaT %if both conditions are binding
                        for ff = 1:size(Z4,2)                           
                            A = diag(diag(Dhat).^(-1/2))*S_q*Lhat*Z4(:,ff); % first term in the minimization problem
                             if A >= 0
                                uO = A;
                                uI = A;
                             else
                                [uO] = fmincon(@(uO)((A-uO)'*BhatO*(A-uO)),[0,0]',-eye(2),[0,0]',Aeq,beq,lb,ub,nonlcon,options); 
                                uI=(A>=0).*A;                                
                             end
                             
                            GbarO(1,ff) = (A-uO)'*BhatO*(A-uO);
                            GbarI(1,ff) = (A-uI)'*BhatI*(A-uI);                           
                        end
                         cvBO(1,s)= prctile(GbarO,(1-alpha1)*100);
                         cvBI(1,s)= prctile(GbarI,(1-alpha1)*100);
                    elseif csi1 < kappaT && csi2 >= kappaT %if only one condition is binding
                         cvBO(1,s) = chi2t;
                         cvBI(1,s) = chi2t;
                    elseif csi1 >= kappaT && csi2 < kappaT %if only one condition is binding
                         cvBO(1,s) = chi2t;
                         cvBI(1,s) = chi2t;   
                    elseif csi1 >= kappaT && csi2 >= kappaT %if no condition is binding
                         cvBO(1,s) = 0;
                         cvBI(1,s) = 0;
                    end
                               
                  % step 2: conditional on q construct the (1-alpha2) confidence set for theta  
                    if Q_RO(1,s) - cvBO(1,s)<= 0
                        CSO(1,ll) = max(0,q1_vec(1,s)*phihatvec(1,1) + q2_vec(1,s)*phihatvec(2,1)-(norminv(1-alpha2/2,0,1))*(Shat(1,1)/T)^(1/2));
                        CSO(2,ll) = q1_vec(1,s)*phihatvec(1,1) + q2_vec(1,s)*phihatvec(2,1) + (norminv(1-alpha2/2,0,1))*(Shat(1,1)/T)^(1/2);
                        CSqO(1,ll)= q1_vec(1,s);
                        arcO(1,ll) = angle(1,s);     
                        ll = ll+1;
                    end
                    
                    if Q_RI(1,s) - cvBI(1,s)<= 0
                        CSI(1,rr) = max(0,q1_vec(1,s)*phihatvec(1,1) + q2_vec(1,s)*phihatvec(2,1)-(norminv(1-alpha2/2,0,1))*(Shat(1,1)/T)^(1/2));
                        CSI(2,rr) = q1_vec(1,s)*phihatvec(1,1) + q2_vec(1,s)*phihatvec(2,1) + (norminv(1-alpha2/2,0,1))*(Shat(1,1)/T)^(1/2);
                        CSqI(1,rr)= q1_vec(1,s);
                        arcI(1,rr) = angle(1,s);
                        rr = rr+1;
                    end                   
                               
            end  %end of loop over angle
            
            
      
        %% step 3 Bonferroni: 1-alpha confidence set for theta  
        CSBO(:,nsim) = [min(CSO(1,:));max(CSO(2,:))];
        CSBI(:,nsim) = [min(CSI(1,:));max(CSI(2,:))];
        CSBqO(:,nsim)= [min(CSqO);max(CSqO)];
        CSBqI(:,nsim)= [min(CSqI);max(CSqI)];
        arc_O(:,nsim) = [min(arcO);max(arcO)];
        arc_I(:,nsim) = [min(arcI);max(arcI)];

        clear  phihatvec mu_hat Q_theta cv_mom1 cv_mom2 CSO CSI G_diffO G_diffI GTpO GTpI CSqO CSqI arcO arcI
        disp(['Elapsed Time: ', num2str(toc)]);
    end %end of loop for nsim

CS_phi= CS_phi/N_sim; %coverage probability for the reduced form parameters
    
    
%coverage probability
temp = (IS_hat(2,:)-(IS(2,1)*ones(1,N_sim)) > 0);
cov_prob = (sum(temp))/N_sim;
lenght = mean(IS_hat(2,:));

tempmomBI = (CSBI(2,:)-(IS(2,1)*ones(1,N_sim)) > 0); %Bonferroni with Identity matrix
cov_probBI = (sum(tempmomBI))/N_sim;
lenghtBI = mean(CSBI(2,:)-CSBI(1,:));

tempmomBO = (CSBO(2,:)-(IS(2,1)*ones(1,N_sim)) > 0); %Bonferroni with Omega-inverse matrix
cov_probBO = (sum(tempmomBO))/N_sim;
lenghtBO = mean(CSBO(2,:)-CSBO(1,:));

tempqO_u = (CSBqO(2,:)-(Fq_u*ones(1,N_sim)) > 0);
cov_probqO_u = (sum(tempqO_u))/N_sim;
tempqO_l = (CSBqO(1,:)-(Fq_l*ones(1,N_sim)) <= 0); %lower endpoint
cov_probqO_l = (sum(tempqO_l))/N_sim;
cov_probqO = min(cov_probqO_u,cov_probqO_l);
lenghtqO = mean(arc_O(2,:)-arc_O(1,:));

tempqI_u = (CSBqI(2,:)-(Fq_u*ones(1,N_sim)) > 0);
cov_probqI_u = (sum(tempqI_u))/N_sim;
tempqI_l = (CSBqI(1,:)-(Fq_l*ones(1,N_sim)) <= 0);%lower endpoint
cov_probqI_l = (sum(tempqI_l))/N_sim;
cov_probqI = min(cov_probqI_u,cov_probqI_l);
lenghtqI = mean(arc_I(2,:)-arc_I(1,:));

Table = [{'Table ','cov prob', 'lenght'};
    'length F(q)',  '_' , num2cell(IS_q(1,2)-IS_q(1,1));
    'length F(theta)', '_', num2cell(IS(2,1));     
    'CSq I',num2cell(cov_probqI),num2cell(lenghtqI);
    'Bonf Iden', num2cell(cov_probBI), num2cell(lenghtBI);
    'CSphi',num2cell(CS_phi), '_';
     ]

save ( ['WorkspaceNobs' num2str(T) 'Nsim' num2str(N_sim) 'design' num2str(design)]) 
