function [ varirf_freq_l, varirf_freq_h ] = construct_irfcs( q_cs, YYirf, ... 
     lambda_hat_all, nlags, nirf, nobs , signrestrictions, nirfsign, alpha2)
% CONSTRUCT CONFIDENCE INTERVAL FOR THETA CONDITIONAL ON q

nq    = size(q_cs,2); %number of vectors q that are included in the CS
nvar  = size(q_cs,1);
k     = 1+nvar*nlags;

% initialization of output matrices
varirf_freq_l = zeros(nirf,nvar);
varirf_freq_h = zeros(nirf,nvar);

% Iterate over variables/horizons                      
for varindx = 1:nvar;
    
    for  horizonindx = 1:nirf;
        
        varirf_temp_l = zeros(nq,1);
        varirf_temp_h = zeros(nq,1);
        
        for   q_ind = 1:nq;
            %  Wald CI conditional on q    
            q = q_cs(:,q_ind);
           
            th_hat = YYirf(horizonindx,(varindx-1)*nvar+1:varindx*nvar)*q;
            th_se  = sqrt( q'*lambda_hat_all(((varindx-1)*nvar+1)+(horizonindx-1)*(nvar^2):(varindx*nvar)+(horizonindx-1)*(nvar^2), ...
                         ((varindx-1)*nvar+1)+(horizonindx-1)*(nvar^2):(varindx*nvar)+(horizonindx-1)*(nvar^2))*q);
            
            varirf_temp_l(q_ind,1) = th_hat - norminv(1-alpha2/2,0,1)*th_se/sqrt(nobs);
            varirf_temp_h(q_ind,1) = th_hat + norminv(1-alpha2/2,0,1)*th_se/sqrt(nobs);
            
        end; 
        
        % take union of CIs
        varirf_freq_l(horizonindx,varindx) = min(varirf_temp_l);
        varirf_freq_h(horizonindx,varindx) = max(varirf_temp_h);
        
        % truncate CI in view of sign restr
        if horizonindx <= nirfsign;
            if signrestrictions(horizonindx,varindx) == -1;
                % response is restricted to be <= 0
                varirf_freq_l(horizonindx,varindx) = min( 0 , varirf_freq_l(horizonindx,varindx) );
                varirf_freq_h(horizonindx,varindx) = min( 0 , varirf_freq_h(horizonindx,varindx) );
            elseif signrestrictions(horizonindx,varindx) == 1;
                % response is restricted to be >= 0
                varirf_freq_l(horizonindx,varindx)= max( 0 , varirf_freq_l(horizonindx,varindx) );
                varirf_freq_h(horizonindx,varindx) = max( 0 , varirf_freq_h(horizonindx,varindx) );
            end;
        end
        
    end; % end of loop for horizonindx
        
end; % end of loop for varindx

end


