function [ YY ] = generate_data( PHI, SIGMA, nobs, nobsinitial)
%This function generates a sample of artificial data from a
%nvar-dimentional VAR(p) with fixed
%parameters PHI and SIGMA
%PHI = [A1';A2';c']

nvar  = size(PHI,2);
nlags = floor(size(PHI,1)/nvar);

if nlags == size(PHI,1)/nvar;
    intercept = 0;
else
    intercept = 1;
end

% generation of normally distributed TxN matrix of N(0,I) errors  
v      = randn((nobs+nobsinitial),nvar);  
SigmaL = chol(SIGMA,'lower');
% generate a TxN matrix of errors 
e      = v*SigmaL';  

Y = zeros((nobs+nobsinitial),nvar);

for t = nlags+1 : nobs + nobsinitial
    if intercept ==1 %model with constant
       Y (t,:) = transpose(PHI'*[reshape(flipud(Y(t-nlags:t-1,:))',nvar*nlags,1); 1]) + e(t,:);
    elseif intercept ==0
       Y (t,:) = transpose(PHI'*(reshape(flipud(Y(t-nlags:t-1,:))',nvar*nlags,1))) + e(t,:);
    end
end

YY = Y(nobsinitial+1:end,:); %delete first nobsinitial observations
end



