%%% procedure to compute the HIGHEST POSTERIOR DENSITY INTERVAL  %%%%%%%
% input: -b_ir: bayesian impulse response function for a particular variable;
%               it's of dimension ndraws x maxH where ndraws is the number of draws over n1xn2 
%               that satisfy the sign restriction and maxH is the maximum
%               horizon
%        -tau: 1-tau confidence interval

function [hpdlow,hpdhigh] = HPDI(b_ir,tau);

[ndraws maxH] = size(b_ir);
hpdlow = zeros(1,maxH);
hpdhigh = zeros(1,maxH);

nwidth = round((1-tau)*ndraws);

i = 1; %horizon i

while i <= maxH
    drawcoli = b_ir(:,i); %select the impulse response at horizon i
    drawcoli = flipud(sort(drawcoli)); %order the element in the vector drawcoli in descending order
    bup = 1;
    minwidth = drawcoli(1,1)-drawcoli(nwidth,1);
    j = 2;
    while j <= (ndraws-nwidth+1)
        newwidth = drawcoli(j,1)-drawcoli(j+nwidth-1,1);
        if newwidth < minwidth
            bup = j;
            minwidth = newwidth;
        end
        j = j+1;
    end %end of while for j
    hpdhigh(1,i) = drawcoli(bup,1);
    hpdlow(1,i) = drawcoli(bup+nwidth-1,1);
    i=i+1;
end %end of while for i 

hpdlow = hpdlow';
hpdhigh = hpdhigh';