
function [LAMBDA_hat_all, LAMBDA_hat_restricted, psi_mean] = estimate_lambda(PHI, SIGMA, ...
                            nrep, nirf, nobs, nobsinitial, signrestrictionsindx);
% this procedure estimates LAMBDA
% LAMBDA_hat_all is cov for all IRFs
% LAMBDA_hat_restricted is cov for sign restricted IRFs

nvar        = size(PHI,2);
nlags       = floor(size(PHI,1)/nvar);
if nlags == size(PHI,1)/nvar;
    intercept = 0;
else
    intercept = 1;
end

% Initialize matrix for simulation output
psi_hat_sim = zeros(nrep,nvar*nvar*nirf);

% Determine number of restricted IRFs
[ YYirf_pop ]              = construct_rfirf(PHI, SIGMA, nirf);
[ YYirfselect ]        = select_irf(YYirf_pop,signrestrictionsindx);
[ psi_restricted_pop ]     = construct_psi_restricted(YYirfselect);

% Initialize matrix for simulation output
psi_hat_sim_restricted = size(psi_restricted_pop,1);

for i=1:nrep;
   % Generate data
   [ YYsim ] =  generate_data( PHI, SIGMA, nobs, nobsinitial);
   % Estimate reduced form parameters for a VAR(p) with nvar variables
   [ PHI_mle, SIGMA_mle ] = estimate_rfvar( YYsim, nlags, intercept );
   
   % generate IRFs up to horizon nirf
   [ YYirf ] = construct_rfirf(PHI_mle, SIGMA_mle, nirf);
   % create column vector of dimention nvar x nirf;
   % includes all responses of all variables to all orthogonalized shocks
   psi_all_sim(i,:) = reshape(YYirf',nvar*nvar*nirf,1);
   
   % select the responses that are restricted
   [YYirfselect]  = select_irf(YYirf,signrestrictionsindx);
   % create column vector, omitting zero responsed due to Cholesky
   [psi_restricted, index_phi] = construct_psi_restricted(YYirfselect);
   psi_restricted_sim(i,:) = psi_restricted';
   
end;

% demean the responses
psi_mean = mean(psi_all_sim);
psi_all_sim        = psi_all_sim-kron(ones(nrep,1),psi_mean);
psi_restricted_sim = psi_restricted_sim-kron(ones(nrep,1),mean(psi_restricted_sim));
% compute asymptotic covariance matrix
LAMBDA_hat_all         = nobs*psi_all_sim'*psi_all_sim/nrep;
LAMBDA_hat_restricted  = nobs*psi_restricted_sim'*psi_restricted_sim/nrep;




